import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.statistics_result_data_type import StatisticsResultDataType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.statistics_metric_model_result import StatisticsMetricModelResult


T = TypeVar("T", bound="StatisticsMetricModel")


@_attrs_define
class StatisticsMetricModel:
    """
    Attributes:
        category (Union[Unset, str]): Statistics category
        collector (Union[Unset, str]): Statistics (metric) name
        sensor_name (Union[Unset, str]): Sensor name
        result_data_type (Union[Unset, StatisticsResultDataType]):
        result (Union[Unset, StatisticsMetricModelResult]): Statistics result for the metric
        collected_at (Union[Unset, datetime.datetime]): The local timestamp when the metric was collected
        executed_at (Union[Unset, int]): The UTC timestamp when the metric was collected (executed)
        sample_count (Union[Unset, int]): The number of the value samples for this result value. Filled only by the
            column value sampling profilers.
        sample_index (Union[Unset, int]): The index of the result that was returned. Filled only by the column value
            sampling profilers to identify each column value sample.
    """

    category: Union[Unset, str] = UNSET
    collector: Union[Unset, str] = UNSET
    sensor_name: Union[Unset, str] = UNSET
    result_data_type: Union[Unset, StatisticsResultDataType] = UNSET
    result: Union[Unset, "StatisticsMetricModelResult"] = UNSET
    collected_at: Union[Unset, datetime.datetime] = UNSET
    executed_at: Union[Unset, int] = UNSET
    sample_count: Union[Unset, int] = UNSET
    sample_index: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        category = self.category
        collector = self.collector
        sensor_name = self.sensor_name
        result_data_type: Union[Unset, str] = UNSET
        if not isinstance(self.result_data_type, Unset):
            result_data_type = self.result_data_type.value

        result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.to_dict()

        collected_at: Union[Unset, str] = UNSET
        if not isinstance(self.collected_at, Unset):
            collected_at = self.collected_at.isoformat()

        executed_at = self.executed_at
        sample_count = self.sample_count
        sample_index = self.sample_index

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if category is not UNSET:
            field_dict["category"] = category
        if collector is not UNSET:
            field_dict["collector"] = collector
        if sensor_name is not UNSET:
            field_dict["sensorName"] = sensor_name
        if result_data_type is not UNSET:
            field_dict["resultDataType"] = result_data_type
        if result is not UNSET:
            field_dict["result"] = result
        if collected_at is not UNSET:
            field_dict["collectedAt"] = collected_at
        if executed_at is not UNSET:
            field_dict["executedAt"] = executed_at
        if sample_count is not UNSET:
            field_dict["sampleCount"] = sample_count
        if sample_index is not UNSET:
            field_dict["sampleIndex"] = sample_index

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.statistics_metric_model_result import StatisticsMetricModelResult

        d = src_dict.copy()
        category = d.pop("category", UNSET)

        collector = d.pop("collector", UNSET)

        sensor_name = d.pop("sensorName", UNSET)

        _result_data_type = d.pop("resultDataType", UNSET)
        result_data_type: Union[Unset, StatisticsResultDataType]
        if isinstance(_result_data_type, Unset):
            result_data_type = UNSET
        else:
            result_data_type = StatisticsResultDataType(_result_data_type)

        _result = d.pop("result", UNSET)
        result: Union[Unset, StatisticsMetricModelResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = StatisticsMetricModelResult.from_dict(_result)

        _collected_at = d.pop("collectedAt", UNSET)
        collected_at: Union[Unset, datetime.datetime]
        if isinstance(_collected_at, Unset):
            collected_at = UNSET
        else:
            collected_at = isoparse(_collected_at)

        executed_at = d.pop("executedAt", UNSET)

        sample_count = d.pop("sampleCount", UNSET)

        sample_index = d.pop("sampleIndex", UNSET)

        statistics_metric_model = cls(
            category=category,
            collector=collector,
            sensor_name=sensor_name,
            result_data_type=result_data_type,
            result=result,
            collected_at=collected_at,
            executed_at=executed_at,
            sample_count=sample_count,
            sample_index=sample_index,
        )

        statistics_metric_model.additional_properties = d
        return statistics_metric_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DataGroupingConfigurationListModel")


@_attrs_define
class DataGroupingConfigurationListModel:
    """Data grouping configurations list model not containing nested objects, but only the name of the grouping
    configuration.

        Attributes:
            connection_name (Union[Unset, str]): Connection name.
            schema_name (Union[Unset, str]): Schema name.
            table_name (Union[Unset, str]): Table name.
            data_grouping_configuration_name (Union[Unset, str]): Data grouping configuration name.
            default_data_grouping_configuration (Union[Unset, bool]): True when this is the default data grouping
                configuration for the table.
            can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
    """

    connection_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, str] = UNSET
    table_name: Union[Unset, str] = UNSET
    data_grouping_configuration_name: Union[Unset, str] = UNSET
    default_data_grouping_configuration: Union[Unset, bool] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        schema_name = self.schema_name
        table_name = self.table_name
        data_grouping_configuration_name = self.data_grouping_configuration_name
        default_data_grouping_configuration = self.default_data_grouping_configuration
        can_edit = self.can_edit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connection_name"] = connection_name
        if schema_name is not UNSET:
            field_dict["schema_name"] = schema_name
        if table_name is not UNSET:
            field_dict["table_name"] = table_name
        if data_grouping_configuration_name is not UNSET:
            field_dict["data_grouping_configuration_name"] = (
                data_grouping_configuration_name
            )
        if default_data_grouping_configuration is not UNSET:
            field_dict["default_data_grouping_configuration"] = (
                default_data_grouping_configuration
            )
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connection_name = d.pop("connection_name", UNSET)

        schema_name = d.pop("schema_name", UNSET)

        table_name = d.pop("table_name", UNSET)

        data_grouping_configuration_name = d.pop(
            "data_grouping_configuration_name", UNSET
        )

        default_data_grouping_configuration = d.pop(
            "default_data_grouping_configuration", UNSET
        )

        can_edit = d.pop("can_edit", UNSET)

        data_grouping_configuration_list_model = cls(
            connection_name=connection_name,
            schema_name=schema_name,
            table_name=table_name,
            data_grouping_configuration_name=data_grouping_configuration_name,
            default_data_grouping_configuration=default_data_grouping_configuration,
            can_edit=can_edit,
        )

        data_grouping_configuration_list_model.additional_properties = d
        return data_grouping_configuration_list_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_daily_partitioned_check_categories_spec import (
        ColumnDailyPartitionedCheckCategoriesSpec,
    )
    from ..models.column_monthly_partitioned_check_categories_spec import (
        ColumnMonthlyPartitionedCheckCategoriesSpec,
    )


T = TypeVar("T", bound="ColumnPartitionedCheckCategoriesSpec")


@_attrs_define
class ColumnPartitionedCheckCategoriesSpec:
    """
    Attributes:
        daily (Union[Unset, ColumnDailyPartitionedCheckCategoriesSpec]):
        monthly (Union[Unset, ColumnMonthlyPartitionedCheckCategoriesSpec]):
    """

    daily: Union[Unset, "ColumnDailyPartitionedCheckCategoriesSpec"] = UNSET
    monthly: Union[Unset, "ColumnMonthlyPartitionedCheckCategoriesSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        daily: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily, Unset):
            daily = self.daily.to_dict()

        monthly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly, Unset):
            monthly = self.monthly.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if daily is not UNSET:
            field_dict["daily"] = daily
        if monthly is not UNSET:
            field_dict["monthly"] = monthly

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_daily_partitioned_check_categories_spec import (
            ColumnDailyPartitionedCheckCategoriesSpec,
        )
        from ..models.column_monthly_partitioned_check_categories_spec import (
            ColumnMonthlyPartitionedCheckCategoriesSpec,
        )

        d = src_dict.copy()
        _daily = d.pop("daily", UNSET)
        daily: Union[Unset, ColumnDailyPartitionedCheckCategoriesSpec]
        if isinstance(_daily, Unset):
            daily = UNSET
        else:
            daily = ColumnDailyPartitionedCheckCategoriesSpec.from_dict(_daily)

        _monthly = d.pop("monthly", UNSET)
        monthly: Union[Unset, ColumnMonthlyPartitionedCheckCategoriesSpec]
        if isinstance(_monthly, Unset):
            monthly = UNSET
        else:
            monthly = ColumnMonthlyPartitionedCheckCategoriesSpec.from_dict(_monthly)

        column_partitioned_check_categories_spec = cls(
            daily=daily,
            monthly=monthly,
        )

        column_partitioned_check_categories_spec.additional_properties = d
        return column_partitioned_check_categories_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.data_type_category import DataTypeCategory
from ..types import UNSET, Unset

T = TypeVar("T", bound="TargetColumnPatternSpec")


@_attrs_define
class TargetColumnPatternSpec:
    """
    Attributes:
        connection (Union[Unset, str]): The data source connection name filter. Accepts wildcards in the format: *conn,
            *, conn*.
        schema (Union[Unset, str]): The schema name filter. Accepts wildcards in the format: *_prod, *, pub*.
        table (Union[Unset, str]): The table name filter. Accepts wildcards in the format: *_customers, *, fact_*.
        stage (Union[Unset, str]): The table stage filter. Accepts wildcards in the format: *_landing, *, staging_*.
        table_priority (Union[Unset, int]): The maximum table priority (inclusive) for tables that are covered by the
            default checks.
        label (Union[Unset, str]): The label filter. Accepts wildcards in the format: *_customers, *, fact_*. The label
            must be present on the connection or table.
        column (Union[Unset, str]): The target column name filter. Accepts wildcards in the format: *id, *, c_*.
        data_type (Union[Unset, str]): The target column data type filter. Filters by a physical (database specific)
            data type name imported from the data source. Accepts wildcards in the format: *int, *, big*.
        data_type_category (Union[Unset, DataTypeCategory]):
    """

    connection: Union[Unset, str] = UNSET
    schema: Union[Unset, str] = UNSET
    table: Union[Unset, str] = UNSET
    stage: Union[Unset, str] = UNSET
    table_priority: Union[Unset, int] = UNSET
    label: Union[Unset, str] = UNSET
    column: Union[Unset, str] = UNSET
    data_type: Union[Unset, str] = UNSET
    data_type_category: Union[Unset, DataTypeCategory] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection = self.connection
        schema = self.schema
        table = self.table
        stage = self.stage
        table_priority = self.table_priority
        label = self.label
        column = self.column
        data_type = self.data_type
        data_type_category: Union[Unset, str] = UNSET
        if not isinstance(self.data_type_category, Unset):
            data_type_category = self.data_type_category.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection is not UNSET:
            field_dict["connection"] = connection
        if schema is not UNSET:
            field_dict["schema"] = schema
        if table is not UNSET:
            field_dict["table"] = table
        if stage is not UNSET:
            field_dict["stage"] = stage
        if table_priority is not UNSET:
            field_dict["table_priority"] = table_priority
        if label is not UNSET:
            field_dict["label"] = label
        if column is not UNSET:
            field_dict["column"] = column
        if data_type is not UNSET:
            field_dict["data_type"] = data_type
        if data_type_category is not UNSET:
            field_dict["data_type_category"] = data_type_category

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connection = d.pop("connection", UNSET)

        schema = d.pop("schema", UNSET)

        table = d.pop("table", UNSET)

        stage = d.pop("stage", UNSET)

        table_priority = d.pop("table_priority", UNSET)

        label = d.pop("label", UNSET)

        column = d.pop("column", UNSET)

        data_type = d.pop("data_type", UNSET)

        _data_type_category = d.pop("data_type_category", UNSET)
        data_type_category: Union[Unset, DataTypeCategory]
        if isinstance(_data_type_category, Unset):
            data_type_category = UNSET
        else:
            data_type_category = DataTypeCategory(_data_type_category)

        target_column_pattern_spec = cls(
            connection=connection,
            schema=schema,
            table=table,
            stage=stage,
            table_priority=table_priority,
            label=label,
            column=column,
            data_type=data_type,
            data_type_category=data_type_category,
        )

        target_column_pattern_spec.additional_properties = d
        return target_column_pattern_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    connection: Union[Unset, None, str] = UNSET,
    schema: Union[Unset, None, str] = UNSET,
    table: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:

    pass

    params: Dict[str, Any] = {}
    params["connection"] = connection

    params["schema"] = schema

    params["table"] = table

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "put",
        "url": "api/datacatalog/sync/pushdatahealth",
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    connection: Union[Unset, None, str] = UNSET,
    schema: Union[Unset, None, str] = UNSET,
    table: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """pushDataQualityStatusToDataCatalog

     Pushes the data quality status of tables matching the search filters to the data catalog.

    Args:
        connection (Union[Unset, None, str]):
        schema (Union[Unset, None, str]):
        table (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        connection=connection,
        schema=schema,
        table=table,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    connection: Union[Unset, None, str] = UNSET,
    schema: Union[Unset, None, str] = UNSET,
    table: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """pushDataQualityStatusToDataCatalog

     Pushes the data quality status of tables matching the search filters to the data catalog.

    Args:
        connection (Union[Unset, None, str]):
        schema (Union[Unset, None, str]):
        table (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        connection=connection,
        schema=schema,
        table=table,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.file_synchronization_direction import FileSynchronizationDirection
from ..types import UNSET, Unset

T = TypeVar("T", bound="SynchronizeMultipleFoldersDqoQueueJobParameters")


@_attrs_define
class SynchronizeMultipleFoldersDqoQueueJobParameters:
    """
    Attributes:
        direction (Union[Unset, FileSynchronizationDirection]):
        force_refresh_native_tables (Union[Unset, bool]): Force full refresh of native tables in the data quality data
            warehouse. The default synchronization mode is to refresh only modified data.
        detect_cron_schedules (Union[Unset, bool]): Scans the yaml files (with the configuration for connections and
            tables) and detects new cron schedules. Detected cron schedules are registered in the cron (Quartz) job
            scheduler.
        sources (Union[Unset, bool]): Synchronize the "sources" folder.
        sensors (Union[Unset, bool]): Synchronize the "sensors" folder.
        rules (Union[Unset, bool]): Synchronize the "rules" folder.
        checks (Union[Unset, bool]): Synchronize the "checks" folder.
        settings (Union[Unset, bool]): Synchronize the "settings" folder.
        credentials (Union[Unset, bool]): Synchronize the ".credentials" folder.
        dictionaries (Union[Unset, bool]): Synchronize the "dictionaries" folder.
        patterns (Union[Unset, bool]): Synchronize the "patterns" folder.
        data_sensor_readouts (Union[Unset, bool]): Synchronize the ".data/sensor_readouts" folder.
        data_check_results (Union[Unset, bool]): Synchronize the ".data/check_results" folder.
        data_statistics (Union[Unset, bool]): Synchronize the ".data/statistics" folder.
        data_errors (Union[Unset, bool]): Synchronize the ".data/errors" folder.
        data_incidents (Union[Unset, bool]): Synchronize the ".data/incidents" folder.
        synchronize_folder_with_local_changes (Union[Unset, bool]): Synchronize all folders that have local changes.
            When this field is set to true, there is no need to enable synchronization of single folders because DQOps will
            decide which folders need synchronization (to be pushed to the cloud).
    """

    direction: Union[Unset, FileSynchronizationDirection] = UNSET
    force_refresh_native_tables: Union[Unset, bool] = UNSET
    detect_cron_schedules: Union[Unset, bool] = UNSET
    sources: Union[Unset, bool] = UNSET
    sensors: Union[Unset, bool] = UNSET
    rules: Union[Unset, bool] = UNSET
    checks: Union[Unset, bool] = UNSET
    settings: Union[Unset, bool] = UNSET
    credentials: Union[Unset, bool] = UNSET
    dictionaries: Union[Unset, bool] = UNSET
    patterns: Union[Unset, bool] = UNSET
    data_sensor_readouts: Union[Unset, bool] = UNSET
    data_check_results: Union[Unset, bool] = UNSET
    data_statistics: Union[Unset, bool] = UNSET
    data_errors: Union[Unset, bool] = UNSET
    data_incidents: Union[Unset, bool] = UNSET
    synchronize_folder_with_local_changes: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        direction: Union[Unset, str] = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        force_refresh_native_tables = self.force_refresh_native_tables
        detect_cron_schedules = self.detect_cron_schedules
        sources = self.sources
        sensors = self.sensors
        rules = self.rules
        checks = self.checks
        settings = self.settings
        credentials = self.credentials
        dictionaries = self.dictionaries
        patterns = self.patterns
        data_sensor_readouts = self.data_sensor_readouts
        data_check_results = self.data_check_results
        data_statistics = self.data_statistics
        data_errors = self.data_errors
        data_incidents = self.data_incidents
        synchronize_folder_with_local_changes = (
            self.synchronize_folder_with_local_changes
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if direction is not UNSET:
            field_dict["direction"] = direction
        if force_refresh_native_tables is not UNSET:
            field_dict["forceRefreshNativeTables"] = force_refresh_native_tables
        if detect_cron_schedules is not UNSET:
            field_dict["detectCronSchedules"] = detect_cron_schedules
        if sources is not UNSET:
            field_dict["sources"] = sources
        if sensors is not UNSET:
            field_dict["sensors"] = sensors
        if rules is not UNSET:
            field_dict["rules"] = rules
        if checks is not UNSET:
            field_dict["checks"] = checks
        if settings is not UNSET:
            field_dict["settings"] = settings
        if credentials is not UNSET:
            field_dict["credentials"] = credentials
        if dictionaries is not UNSET:
            field_dict["dictionaries"] = dictionaries
        if patterns is not UNSET:
            field_dict["patterns"] = patterns
        if data_sensor_readouts is not UNSET:
            field_dict["dataSensorReadouts"] = data_sensor_readouts
        if data_check_results is not UNSET:
            field_dict["dataCheckResults"] = data_check_results
        if data_statistics is not UNSET:
            field_dict["dataStatistics"] = data_statistics
        if data_errors is not UNSET:
            field_dict["dataErrors"] = data_errors
        if data_incidents is not UNSET:
            field_dict["dataIncidents"] = data_incidents
        if synchronize_folder_with_local_changes is not UNSET:
            field_dict["synchronizeFolderWithLocalChanges"] = (
                synchronize_folder_with_local_changes
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _direction = d.pop("direction", UNSET)
        direction: Union[Unset, FileSynchronizationDirection]
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = FileSynchronizationDirection(_direction)

        force_refresh_native_tables = d.pop("forceRefreshNativeTables", UNSET)

        detect_cron_schedules = d.pop("detectCronSchedules", UNSET)

        sources = d.pop("sources", UNSET)

        sensors = d.pop("sensors", UNSET)

        rules = d.pop("rules", UNSET)

        checks = d.pop("checks", UNSET)

        settings = d.pop("settings", UNSET)

        credentials = d.pop("credentials", UNSET)

        dictionaries = d.pop("dictionaries", UNSET)

        patterns = d.pop("patterns", UNSET)

        data_sensor_readouts = d.pop("dataSensorReadouts", UNSET)

        data_check_results = d.pop("dataCheckResults", UNSET)

        data_statistics = d.pop("dataStatistics", UNSET)

        data_errors = d.pop("dataErrors", UNSET)

        data_incidents = d.pop("dataIncidents", UNSET)

        synchronize_folder_with_local_changes = d.pop(
            "synchronizeFolderWithLocalChanges", UNSET
        )

        synchronize_multiple_folders_dqo_queue_job_parameters = cls(
            direction=direction,
            force_refresh_native_tables=force_refresh_native_tables,
            detect_cron_schedules=detect_cron_schedules,
            sources=sources,
            sensors=sensors,
            rules=rules,
            checks=checks,
            settings=settings,
            credentials=credentials,
            dictionaries=dictionaries,
            patterns=patterns,
            data_sensor_readouts=data_sensor_readouts,
            data_check_results=data_check_results,
            data_statistics=data_statistics,
            data_errors=data_errors,
            data_incidents=data_incidents,
            synchronize_folder_with_local_changes=synchronize_folder_with_local_changes,
        )

        synchronize_multiple_folders_dqo_queue_job_parameters.additional_properties = d
        return synchronize_multiple_folders_dqo_queue_job_parameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_monitoring_check_categories_spec import (
        ColumnMonitoringCheckCategoriesSpec,
    )
    from ..models.column_partitioned_check_categories_spec import (
        ColumnPartitionedCheckCategoriesSpec,
    )
    from ..models.column_profiling_check_categories_spec import (
        ColumnProfilingCheckCategoriesSpec,
    )
    from ..models.column_spec_advanced_properties import ColumnSpecAdvancedProperties
    from ..models.column_statistics_collectors_root_categories_spec import (
        ColumnStatisticsCollectorsRootCategoriesSpec,
    )
    from ..models.column_type_snapshot_spec import ColumnTypeSnapshotSpec
    from ..models.comment_spec import CommentSpec


T = TypeVar("T", bound="ColumnSpec")


@_attrs_define
class ColumnSpec:
    """
    Attributes:
        disabled (Union[Unset, bool]): Disables all data quality checks on the column. Data quality checks will not be
            executed.
        sql_expression (Union[Unset, str]): SQL expression used for calculated fields or when additional column value
            transformation is required before the column can be used for analysis with data quality checks (data type
            conversion, transformation). It should be an SQL expression that uses the SQL language of the analyzed database
            type. Use the replacement tokens {table} to replace the content with the full table name, {alias} to replace the
            content with the table alias of the table under analysis, or {column} to replace the content with the analyzed
            column name. An example of extracting a value from a string column storing JSON in PostgreSQL:
            "{column}::json->'address'->'zip'".
        type_snapshot (Union[Unset, ColumnTypeSnapshotSpec]):
        id (Union[Unset, bool]): True when this column is a part of the primary key or a business key that identifies a
            row. Error sampling captures values of id columns to identify the row where the error sample was found.
        profiling_checks (Union[Unset, ColumnProfilingCheckCategoriesSpec]):
        monitoring_checks (Union[Unset, ColumnMonitoringCheckCategoriesSpec]):
        partitioned_checks (Union[Unset, ColumnPartitionedCheckCategoriesSpec]):
        statistics (Union[Unset, ColumnStatisticsCollectorsRootCategoriesSpec]):
        labels (Union[Unset, List[str]]): Custom labels that were assigned to the column. Labels are used for searching
            for columns when filtered data quality checks are executed.
        comments (Union[Unset, List['CommentSpec']]): Comments for change tracking. Please put comments in this
            collection because YAML comments may be removed when the YAML file is modified by the tool (serialization and
            deserialization will remove non tracked comments).
        advanced_properties (Union[Unset, ColumnSpecAdvancedProperties]): A dictionary of advanced properties that can
            be used for e.g. to support mapping data to data catalogs, a key/value dictionary.
    """

    disabled: Union[Unset, bool] = UNSET
    sql_expression: Union[Unset, str] = UNSET
    type_snapshot: Union[Unset, "ColumnTypeSnapshotSpec"] = UNSET
    id: Union[Unset, bool] = UNSET
    profiling_checks: Union[Unset, "ColumnProfilingCheckCategoriesSpec"] = UNSET
    monitoring_checks: Union[Unset, "ColumnMonitoringCheckCategoriesSpec"] = UNSET
    partitioned_checks: Union[Unset, "ColumnPartitionedCheckCategoriesSpec"] = UNSET
    statistics: Union[Unset, "ColumnStatisticsCollectorsRootCategoriesSpec"] = UNSET
    labels: Union[Unset, List[str]] = UNSET
    comments: Union[Unset, List["CommentSpec"]] = UNSET
    advanced_properties: Union[Unset, "ColumnSpecAdvancedProperties"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        disabled = self.disabled
        sql_expression = self.sql_expression
        type_snapshot: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.type_snapshot, Unset):
            type_snapshot = self.type_snapshot.to_dict()

        id = self.id
        profiling_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profiling_checks, Unset):
            profiling_checks = self.profiling_checks.to_dict()

        monitoring_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monitoring_checks, Unset):
            monitoring_checks = self.monitoring_checks.to_dict()

        partitioned_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.partitioned_checks, Unset):
            partitioned_checks = self.partitioned_checks.to_dict()

        statistics: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.statistics, Unset):
            statistics = self.statistics.to_dict()

        labels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels

        comments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.comments, Unset):
            comments = []
            for comments_item_data in self.comments:
                comments_item = comments_item_data.to_dict()

                comments.append(comments_item)

        advanced_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.advanced_properties, Unset):
            advanced_properties = self.advanced_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if sql_expression is not UNSET:
            field_dict["sql_expression"] = sql_expression
        if type_snapshot is not UNSET:
            field_dict["type_snapshot"] = type_snapshot
        if id is not UNSET:
            field_dict["id"] = id
        if profiling_checks is not UNSET:
            field_dict["profiling_checks"] = profiling_checks
        if monitoring_checks is not UNSET:
            field_dict["monitoring_checks"] = monitoring_checks
        if partitioned_checks is not UNSET:
            field_dict["partitioned_checks"] = partitioned_checks
        if statistics is not UNSET:
            field_dict["statistics"] = statistics
        if labels is not UNSET:
            field_dict["labels"] = labels
        if comments is not UNSET:
            field_dict["comments"] = comments
        if advanced_properties is not UNSET:
            field_dict["advanced_properties"] = advanced_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_monitoring_check_categories_spec import (
            ColumnMonitoringCheckCategoriesSpec,
        )
        from ..models.column_partitioned_check_categories_spec import (
            ColumnPartitionedCheckCategoriesSpec,
        )
        from ..models.column_profiling_check_categories_spec import (
            ColumnProfilingCheckCategoriesSpec,
        )
        from ..models.column_spec_advanced_properties import (
            ColumnSpecAdvancedProperties,
        )
        from ..models.column_statistics_collectors_root_categories_spec import (
            ColumnStatisticsCollectorsRootCategoriesSpec,
        )
        from ..models.column_type_snapshot_spec import ColumnTypeSnapshotSpec
        from ..models.comment_spec import CommentSpec

        d = src_dict.copy()
        disabled = d.pop("disabled", UNSET)

        sql_expression = d.pop("sql_expression", UNSET)

        _type_snapshot = d.pop("type_snapshot", UNSET)
        type_snapshot: Union[Unset, ColumnTypeSnapshotSpec]
        if isinstance(_type_snapshot, Unset):
            type_snapshot = UNSET
        else:
            type_snapshot = ColumnTypeSnapshotSpec.from_dict(_type_snapshot)

        id = d.pop("id", UNSET)

        _profiling_checks = d.pop("profiling_checks", UNSET)
        profiling_checks: Union[Unset, ColumnProfilingCheckCategoriesSpec]
        if isinstance(_profiling_checks, Unset):
            profiling_checks = UNSET
        else:
            profiling_checks = ColumnProfilingCheckCategoriesSpec.from_dict(
                _profiling_checks
            )

        _monitoring_checks = d.pop("monitoring_checks", UNSET)
        monitoring_checks: Union[Unset, ColumnMonitoringCheckCategoriesSpec]
        if isinstance(_monitoring_checks, Unset):
            monitoring_checks = UNSET
        else:
            monitoring_checks = ColumnMonitoringCheckCategoriesSpec.from_dict(
                _monitoring_checks
            )

        _partitioned_checks = d.pop("partitioned_checks", UNSET)
        partitioned_checks: Union[Unset, ColumnPartitionedCheckCategoriesSpec]
        if isinstance(_partitioned_checks, Unset):
            partitioned_checks = UNSET
        else:
            partitioned_checks = ColumnPartitionedCheckCategoriesSpec.from_dict(
                _partitioned_checks
            )

        _statistics = d.pop("statistics", UNSET)
        statistics: Union[Unset, ColumnStatisticsCollectorsRootCategoriesSpec]
        if isinstance(_statistics, Unset):
            statistics = UNSET
        else:
            statistics = ColumnStatisticsCollectorsRootCategoriesSpec.from_dict(
                _statistics
            )

        labels = cast(List[str], d.pop("labels", UNSET))

        comments = []
        _comments = d.pop("comments", UNSET)
        for comments_item_data in _comments or []:
            comments_item = CommentSpec.from_dict(comments_item_data)

            comments.append(comments_item)

        _advanced_properties = d.pop("advanced_properties", UNSET)
        advanced_properties: Union[Unset, ColumnSpecAdvancedProperties]
        if isinstance(_advanced_properties, Unset):
            advanced_properties = UNSET
        else:
            advanced_properties = ColumnSpecAdvancedProperties.from_dict(
                _advanced_properties
            )

        column_spec = cls(
            disabled=disabled,
            sql_expression=sql_expression,
            type_snapshot=type_snapshot,
            id=id,
            profiling_checks=profiling_checks,
            monitoring_checks=monitoring_checks,
            partitioned_checks=partitioned_checks,
            statistics=statistics,
            labels=labels,
            comments=comments,
            advanced_properties=advanced_properties,
        )

        column_spec.additional_properties = d
        return column_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DqoRoot(str, Enum):
    CHECKS = "checks"
    CREDENTIALS = "credentials"
    DATA_CHECK_RESULTS = "data_check_results"
    DATA_ERRORS = "data_errors"
    DATA_ERROR_SAMPLES = "data_error_samples"
    DATA_INCIDENTS = "data_incidents"
    DATA_SENSOR_READOUTS = "data_sensor_readouts"
    DATA_STATISTICS = "data_statistics"
    DICTIONARIES = "dictionaries"
    PATTERNS = "patterns"
    RULES = "rules"
    SENSORS = "sensors"
    SETTINGS = "settings"
    SOURCES = "sources"
    VALUE_14 = "_indexes"
    VALUE_15 = "_indexes_sources"
    VALUE_16 = "_local_settings"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class HistoricDataPointsGrouping(str, Enum):
    DAY = "day"
    HOUR = "hour"
    LAST_N_READOUTS = "last_n_readouts"
    MONTH = "month"
    QUARTER = "quarter"
    WEEK = "week"
    YEAR = "year"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CheckListModel")


@_attrs_define
class CheckListModel:
    """Simplistic model that returns a single data quality check, its name and "configured" flag

    Attributes:
        check_category (Union[Unset, str]): Check category.
        check_name (Union[Unset, str]): Data quality check name that is used in YAML.
        help_text (Union[Unset, str]): Help text that describes the data quality check.
        configured (Union[Unset, bool]): True if the data quality check is configured (not null). When saving the data
            quality check configuration, set the flag to true for storing the check.
    """

    check_category: Union[Unset, str] = UNSET
    check_name: Union[Unset, str] = UNSET
    help_text: Union[Unset, str] = UNSET
    configured: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        check_category = self.check_category
        check_name = self.check_name
        help_text = self.help_text
        configured = self.configured

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if check_category is not UNSET:
            field_dict["check_category"] = check_category
        if check_name is not UNSET:
            field_dict["check_name"] = check_name
        if help_text is not UNSET:
            field_dict["help_text"] = help_text
        if configured is not UNSET:
            field_dict["configured"] = configured

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        check_category = d.pop("check_category", UNSET)

        check_name = d.pop("check_name", UNSET)

        help_text = d.pop("help_text", UNSET)

        configured = d.pop("configured", UNSET)

        check_list_model = cls(
            check_category=check_category,
            check_name=check_name,
            help_text=help_text,
            configured=configured,
        )

        check_list_model.additional_properties = d
        return check_list_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.collect_error_samples_on_table_parameters import (
        CollectErrorSamplesOnTableParameters,
    )
    from ..models.collect_error_samples_parameters import CollectErrorSamplesParameters
    from ..models.collect_statistics_on_table_queue_job_parameters import (
        CollectStatisticsOnTableQueueJobParameters,
    )
    from ..models.collect_statistics_queue_job_parameters import (
        CollectStatisticsQueueJobParameters,
    )
    from ..models.cron_schedule_spec import CronScheduleSpec
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )
    from ..models.import_schema_queue_job_parameters import (
        ImportSchemaQueueJobParameters,
    )
    from ..models.import_tables_queue_job_parameters import (
        ImportTablesQueueJobParameters,
    )
    from ..models.repair_stored_data_queue_job_parameters import (
        RepairStoredDataQueueJobParameters,
    )
    from ..models.run_checks_on_table_parameters import RunChecksOnTableParameters
    from ..models.run_checks_parameters import RunChecksParameters
    from ..models.synchronize_multiple_folders_dqo_queue_job_parameters import (
        SynchronizeMultipleFoldersDqoQueueJobParameters,
    )
    from ..models.synchronize_root_folder_dqo_queue_job_parameters import (
        SynchronizeRootFolderDqoQueueJobParameters,
    )


T = TypeVar("T", bound="DqoJobEntryParametersModel")


@_attrs_define
class DqoJobEntryParametersModel:
    """
    Attributes:
        synchronize_root_folder_parameters (Union[Unset, SynchronizeRootFolderDqoQueueJobParameters]):
        synchronize_multiple_folders_parameters (Union[Unset, SynchronizeMultipleFoldersDqoQueueJobParameters]):
        run_scheduled_checks_parameters (Union[Unset, CronScheduleSpec]):
        collect_scheduled_statistics_parameters (Union[Unset, CronScheduleSpec]):
        auto_import_tables_parameters (Union[Unset, CronScheduleSpec]):
        run_checks_parameters (Union[Unset, RunChecksParameters]): Run checks configuration, specifies the target checks
            that should be executed and an optional time window.
        run_checks_on_table_parameters (Union[Unset, RunChecksOnTableParameters]): Run checks configuration for a job
            that will run checks on a single table, specifies the target table and the target checks that should be executed
            and an optional time window.
        collect_statistics_parameters (Union[Unset, CollectStatisticsQueueJobParameters]):
        collect_statistics_on_table_parameters (Union[Unset, CollectStatisticsOnTableQueueJobParameters]):
        collect_error_samples_parameters (Union[Unset, CollectErrorSamplesParameters]): Collect error samples job
            parameters, specifies the target checks that should be executed to collect error samples and an optional time
            window.
        collect_error_samples_on_table_parameters (Union[Unset, CollectErrorSamplesOnTableParameters]):
        import_schema_parameters (Union[Unset, ImportSchemaQueueJobParameters]):
        import_table_parameters (Union[Unset, ImportTablesQueueJobParameters]):
        delete_stored_data_parameters (Union[Unset, DeleteStoredDataQueueJobParameters]):
        repair_stored_data_parameters (Union[Unset, RepairStoredDataQueueJobParameters]):
    """

    synchronize_root_folder_parameters: Union[
        Unset, "SynchronizeRootFolderDqoQueueJobParameters"
    ] = UNSET
    synchronize_multiple_folders_parameters: Union[
        Unset, "SynchronizeMultipleFoldersDqoQueueJobParameters"
    ] = UNSET
    run_scheduled_checks_parameters: Union[Unset, "CronScheduleSpec"] = UNSET
    collect_scheduled_statistics_parameters: Union[Unset, "CronScheduleSpec"] = UNSET
    auto_import_tables_parameters: Union[Unset, "CronScheduleSpec"] = UNSET
    run_checks_parameters: Union[Unset, "RunChecksParameters"] = UNSET
    run_checks_on_table_parameters: Union[Unset, "RunChecksOnTableParameters"] = UNSET
    collect_statistics_parameters: Union[
        Unset, "CollectStatisticsQueueJobParameters"
    ] = UNSET
    collect_statistics_on_table_parameters: Union[
        Unset, "CollectStatisticsOnTableQueueJobParameters"
    ] = UNSET
    collect_error_samples_parameters: Union[Unset, "CollectErrorSamplesParameters"] = (
        UNSET
    )
    collect_error_samples_on_table_parameters: Union[
        Unset, "CollectErrorSamplesOnTableParameters"
    ] = UNSET
    import_schema_parameters: Union[Unset, "ImportSchemaQueueJobParameters"] = UNSET
    import_table_parameters: Union[Unset, "ImportTablesQueueJobParameters"] = UNSET
    delete_stored_data_parameters: Union[
        Unset, "DeleteStoredDataQueueJobParameters"
    ] = UNSET
    repair_stored_data_parameters: Union[
        Unset, "RepairStoredDataQueueJobParameters"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        synchronize_root_folder_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.synchronize_root_folder_parameters, Unset):
            synchronize_root_folder_parameters = (
                self.synchronize_root_folder_parameters.to_dict()
            )

        synchronize_multiple_folders_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.synchronize_multiple_folders_parameters, Unset):
            synchronize_multiple_folders_parameters = (
                self.synchronize_multiple_folders_parameters.to_dict()
            )

        run_scheduled_checks_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_scheduled_checks_parameters, Unset):
            run_scheduled_checks_parameters = (
                self.run_scheduled_checks_parameters.to_dict()
            )

        collect_scheduled_statistics_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_scheduled_statistics_parameters, Unset):
            collect_scheduled_statistics_parameters = (
                self.collect_scheduled_statistics_parameters.to_dict()
            )

        auto_import_tables_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.auto_import_tables_parameters, Unset):
            auto_import_tables_parameters = self.auto_import_tables_parameters.to_dict()

        run_checks_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_parameters, Unset):
            run_checks_parameters = self.run_checks_parameters.to_dict()

        run_checks_on_table_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_on_table_parameters, Unset):
            run_checks_on_table_parameters = (
                self.run_checks_on_table_parameters.to_dict()
            )

        collect_statistics_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_parameters, Unset):
            collect_statistics_parameters = self.collect_statistics_parameters.to_dict()

        collect_statistics_on_table_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_on_table_parameters, Unset):
            collect_statistics_on_table_parameters = (
                self.collect_statistics_on_table_parameters.to_dict()
            )

        collect_error_samples_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_error_samples_parameters, Unset):
            collect_error_samples_parameters = (
                self.collect_error_samples_parameters.to_dict()
            )

        collect_error_samples_on_table_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_error_samples_on_table_parameters, Unset):
            collect_error_samples_on_table_parameters = (
                self.collect_error_samples_on_table_parameters.to_dict()
            )

        import_schema_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.import_schema_parameters, Unset):
            import_schema_parameters = self.import_schema_parameters.to_dict()

        import_table_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.import_table_parameters, Unset):
            import_table_parameters = self.import_table_parameters.to_dict()

        delete_stored_data_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.delete_stored_data_parameters, Unset):
            delete_stored_data_parameters = self.delete_stored_data_parameters.to_dict()

        repair_stored_data_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.repair_stored_data_parameters, Unset):
            repair_stored_data_parameters = self.repair_stored_data_parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if synchronize_root_folder_parameters is not UNSET:
            field_dict["synchronizeRootFolderParameters"] = (
                synchronize_root_folder_parameters
            )
        if synchronize_multiple_folders_parameters is not UNSET:
            field_dict["synchronizeMultipleFoldersParameters"] = (
                synchronize_multiple_folders_parameters
            )
        if run_scheduled_checks_parameters is not UNSET:
            field_dict["runScheduledChecksParameters"] = run_scheduled_checks_parameters
        if collect_scheduled_statistics_parameters is not UNSET:
            field_dict["collectScheduledStatisticsParameters"] = (
                collect_scheduled_statistics_parameters
            )
        if auto_import_tables_parameters is not UNSET:
            field_dict["autoImportTablesParameters"] = auto_import_tables_parameters
        if run_checks_parameters is not UNSET:
            field_dict["runChecksParameters"] = run_checks_parameters
        if run_checks_on_table_parameters is not UNSET:
            field_dict["runChecksOnTableParameters"] = run_checks_on_table_parameters
        if collect_statistics_parameters is not UNSET:
            field_dict["collectStatisticsParameters"] = collect_statistics_parameters
        if collect_statistics_on_table_parameters is not UNSET:
            field_dict["collectStatisticsOnTableParameters"] = (
                collect_statistics_on_table_parameters
            )
        if collect_error_samples_parameters is not UNSET:
            field_dict["collectErrorSamplesParameters"] = (
                collect_error_samples_parameters
            )
        if collect_error_samples_on_table_parameters is not UNSET:
            field_dict["collectErrorSamplesOnTableParameters"] = (
                collect_error_samples_on_table_parameters
            )
        if import_schema_parameters is not UNSET:
            field_dict["importSchemaParameters"] = import_schema_parameters
        if import_table_parameters is not UNSET:
            field_dict["importTableParameters"] = import_table_parameters
        if delete_stored_data_parameters is not UNSET:
            field_dict["deleteStoredDataParameters"] = delete_stored_data_parameters
        if repair_stored_data_parameters is not UNSET:
            field_dict["repairStoredDataParameters"] = repair_stored_data_parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.collect_error_samples_on_table_parameters import (
            CollectErrorSamplesOnTableParameters,
        )
        from ..models.collect_error_samples_parameters import (
            CollectErrorSamplesParameters,
        )
        from ..models.collect_statistics_on_table_queue_job_parameters import (
            CollectStatisticsOnTableQueueJobParameters,
        )
        from ..models.collect_statistics_queue_job_parameters import (
            CollectStatisticsQueueJobParameters,
        )
        from ..models.cron_schedule_spec import CronScheduleSpec
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )
        from ..models.import_schema_queue_job_parameters import (
            ImportSchemaQueueJobParameters,
        )
        from ..models.import_tables_queue_job_parameters import (
            ImportTablesQueueJobParameters,
        )
        from ..models.repair_stored_data_queue_job_parameters import (
            RepairStoredDataQueueJobParameters,
        )
        from ..models.run_checks_on_table_parameters import RunChecksOnTableParameters
        from ..models.run_checks_parameters import RunChecksParameters
        from ..models.synchronize_multiple_folders_dqo_queue_job_parameters import (
            SynchronizeMultipleFoldersDqoQueueJobParameters,
        )
        from ..models.synchronize_root_folder_dqo_queue_job_parameters import (
            SynchronizeRootFolderDqoQueueJobParameters,
        )

        d = src_dict.copy()
        _synchronize_root_folder_parameters = d.pop(
            "synchronizeRootFolderParameters", UNSET
        )
        synchronize_root_folder_parameters: Union[
            Unset, SynchronizeRootFolderDqoQueueJobParameters
        ]
        if isinstance(_synchronize_root_folder_parameters, Unset):
            synchronize_root_folder_parameters = UNSET
        else:
            synchronize_root_folder_parameters = (
                SynchronizeRootFolderDqoQueueJobParameters.from_dict(
                    _synchronize_root_folder_parameters
                )
            )

        _synchronize_multiple_folders_parameters = d.pop(
            "synchronizeMultipleFoldersParameters", UNSET
        )
        synchronize_multiple_folders_parameters: Union[
            Unset, SynchronizeMultipleFoldersDqoQueueJobParameters
        ]
        if isinstance(_synchronize_multiple_folders_parameters, Unset):
            synchronize_multiple_folders_parameters = UNSET
        else:
            synchronize_multiple_folders_parameters = (
                SynchronizeMultipleFoldersDqoQueueJobParameters.from_dict(
                    _synchronize_multiple_folders_parameters
                )
            )

        _run_scheduled_checks_parameters = d.pop("runScheduledChecksParameters", UNSET)
        run_scheduled_checks_parameters: Union[Unset, CronScheduleSpec]
        if isinstance(_run_scheduled_checks_parameters, Unset):
            run_scheduled_checks_parameters = UNSET
        else:
            run_scheduled_checks_parameters = CronScheduleSpec.from_dict(
                _run_scheduled_checks_parameters
            )

        _collect_scheduled_statistics_parameters = d.pop(
            "collectScheduledStatisticsParameters", UNSET
        )
        collect_scheduled_statistics_parameters: Union[Unset, CronScheduleSpec]
        if isinstance(_collect_scheduled_statistics_parameters, Unset):
            collect_scheduled_statistics_parameters = UNSET
        else:
            collect_scheduled_statistics_parameters = CronScheduleSpec.from_dict(
                _collect_scheduled_statistics_parameters
            )

        _auto_import_tables_parameters = d.pop("autoImportTablesParameters", UNSET)
        auto_import_tables_parameters: Union[Unset, CronScheduleSpec]
        if isinstance(_auto_import_tables_parameters, Unset):
            auto_import_tables_parameters = UNSET
        else:
            auto_import_tables_parameters = CronScheduleSpec.from_dict(
                _auto_import_tables_parameters
            )

        _run_checks_parameters = d.pop("runChecksParameters", UNSET)
        run_checks_parameters: Union[Unset, RunChecksParameters]
        if isinstance(_run_checks_parameters, Unset):
            run_checks_parameters = UNSET
        else:
            run_checks_parameters = RunChecksParameters.from_dict(
                _run_checks_parameters
            )

        _run_checks_on_table_parameters = d.pop("runChecksOnTableParameters", UNSET)
        run_checks_on_table_parameters: Union[Unset, RunChecksOnTableParameters]
        if isinstance(_run_checks_on_table_parameters, Unset):
            run_checks_on_table_parameters = UNSET
        else:
            run_checks_on_table_parameters = RunChecksOnTableParameters.from_dict(
                _run_checks_on_table_parameters
            )

        _collect_statistics_parameters = d.pop("collectStatisticsParameters", UNSET)
        collect_statistics_parameters: Union[Unset, CollectStatisticsQueueJobParameters]
        if isinstance(_collect_statistics_parameters, Unset):
            collect_statistics_parameters = UNSET
        else:
            collect_statistics_parameters = (
                CollectStatisticsQueueJobParameters.from_dict(
                    _collect_statistics_parameters
                )
            )

        _collect_statistics_on_table_parameters = d.pop(
            "collectStatisticsOnTableParameters", UNSET
        )
        collect_statistics_on_table_parameters: Union[
            Unset, CollectStatisticsOnTableQueueJobParameters
        ]
        if isinstance(_collect_statistics_on_table_parameters, Unset):
            collect_statistics_on_table_parameters = UNSET
        else:
            collect_statistics_on_table_parameters = (
                CollectStatisticsOnTableQueueJobParameters.from_dict(
                    _collect_statistics_on_table_parameters
                )
            )

        _collect_error_samples_parameters = d.pop(
            "collectErrorSamplesParameters", UNSET
        )
        collect_error_samples_parameters: Union[Unset, CollectErrorSamplesParameters]
        if isinstance(_collect_error_samples_parameters, Unset):
            collect_error_samples_parameters = UNSET
        else:
            collect_error_samples_parameters = CollectErrorSamplesParameters.from_dict(
                _collect_error_samples_parameters
            )

        _collect_error_samples_on_table_parameters = d.pop(
            "collectErrorSamplesOnTableParameters", UNSET
        )
        collect_error_samples_on_table_parameters: Union[
            Unset, CollectErrorSamplesOnTableParameters
        ]
        if isinstance(_collect_error_samples_on_table_parameters, Unset):
            collect_error_samples_on_table_parameters = UNSET
        else:
            collect_error_samples_on_table_parameters = (
                CollectErrorSamplesOnTableParameters.from_dict(
                    _collect_error_samples_on_table_parameters
                )
            )

        _import_schema_parameters = d.pop("importSchemaParameters", UNSET)
        import_schema_parameters: Union[Unset, ImportSchemaQueueJobParameters]
        if isinstance(_import_schema_parameters, Unset):
            import_schema_parameters = UNSET
        else:
            import_schema_parameters = ImportSchemaQueueJobParameters.from_dict(
                _import_schema_parameters
            )

        _import_table_parameters = d.pop("importTableParameters", UNSET)
        import_table_parameters: Union[Unset, ImportTablesQueueJobParameters]
        if isinstance(_import_table_parameters, Unset):
            import_table_parameters = UNSET
        else:
            import_table_parameters = ImportTablesQueueJobParameters.from_dict(
                _import_table_parameters
            )

        _delete_stored_data_parameters = d.pop("deleteStoredDataParameters", UNSET)
        delete_stored_data_parameters: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_delete_stored_data_parameters, Unset):
            delete_stored_data_parameters = UNSET
        else:
            delete_stored_data_parameters = (
                DeleteStoredDataQueueJobParameters.from_dict(
                    _delete_stored_data_parameters
                )
            )

        _repair_stored_data_parameters = d.pop("repairStoredDataParameters", UNSET)
        repair_stored_data_parameters: Union[Unset, RepairStoredDataQueueJobParameters]
        if isinstance(_repair_stored_data_parameters, Unset):
            repair_stored_data_parameters = UNSET
        else:
            repair_stored_data_parameters = (
                RepairStoredDataQueueJobParameters.from_dict(
                    _repair_stored_data_parameters
                )
            )

        dqo_job_entry_parameters_model = cls(
            synchronize_root_folder_parameters=synchronize_root_folder_parameters,
            synchronize_multiple_folders_parameters=synchronize_multiple_folders_parameters,
            run_scheduled_checks_parameters=run_scheduled_checks_parameters,
            collect_scheduled_statistics_parameters=collect_scheduled_statistics_parameters,
            auto_import_tables_parameters=auto_import_tables_parameters,
            run_checks_parameters=run_checks_parameters,
            run_checks_on_table_parameters=run_checks_on_table_parameters,
            collect_statistics_parameters=collect_statistics_parameters,
            collect_statistics_on_table_parameters=collect_statistics_on_table_parameters,
            collect_error_samples_parameters=collect_error_samples_parameters,
            collect_error_samples_on_table_parameters=collect_error_samples_on_table_parameters,
            import_schema_parameters=import_schema_parameters,
            import_table_parameters=import_table_parameters,
            delete_stored_data_parameters=delete_stored_data_parameters,
            repair_stored_data_parameters=repair_stored_data_parameters,
        )

        dqo_job_entry_parameters_model.additional_properties = d
        return dqo_job_entry_parameters_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.display_hint import DisplayHint
from ..models.parameter_data_type import ParameterDataType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ParameterDefinitionSpec")


@_attrs_define
class ParameterDefinitionSpec:
    """Defines a single field that is a sensor parameter or a rule parameter.

    Attributes:
        field_name (Union[Unset, str]): Field name that matches the field name (snake_case) used in the YAML
            specification.
        display_name (Union[Unset, str]): Field display name that should be shown as a label for the control.
        help_text (Union[Unset, str]): Help text (full description) that will be shown to the user as a hint when the
            cursor is moved over the control.
        data_type (Union[Unset, ParameterDataType]):
        display_hint (Union[Unset, DisplayHint]):
        required (Union[Unset, bool]): True when the value for the parameter must be provided.
        allowed_values (Union[Unset, List[str]]): List of allowed values for a field that is of an enum type.
        default_value (Union[Unset, str]): The default value for a parameter in a custom check or a custom rule.
    """

    field_name: Union[Unset, str] = UNSET
    display_name: Union[Unset, str] = UNSET
    help_text: Union[Unset, str] = UNSET
    data_type: Union[Unset, ParameterDataType] = UNSET
    display_hint: Union[Unset, DisplayHint] = UNSET
    required: Union[Unset, bool] = UNSET
    allowed_values: Union[Unset, List[str]] = UNSET
    default_value: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_name = self.field_name
        display_name = self.display_name
        help_text = self.help_text
        data_type: Union[Unset, str] = UNSET
        if not isinstance(self.data_type, Unset):
            data_type = self.data_type.value

        display_hint: Union[Unset, str] = UNSET
        if not isinstance(self.display_hint, Unset):
            display_hint = self.display_hint.value

        required = self.required
        allowed_values: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_values, Unset):
            allowed_values = self.allowed_values

        default_value = self.default_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_name is not UNSET:
            field_dict["field_name"] = field_name
        if display_name is not UNSET:
            field_dict["display_name"] = display_name
        if help_text is not UNSET:
            field_dict["help_text"] = help_text
        if data_type is not UNSET:
            field_dict["data_type"] = data_type
        if display_hint is not UNSET:
            field_dict["display_hint"] = display_hint
        if required is not UNSET:
            field_dict["required"] = required
        if allowed_values is not UNSET:
            field_dict["allowed_values"] = allowed_values
        if default_value is not UNSET:
            field_dict["default_value"] = default_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field_name = d.pop("field_name", UNSET)

        display_name = d.pop("display_name", UNSET)

        help_text = d.pop("help_text", UNSET)

        _data_type = d.pop("data_type", UNSET)
        data_type: Union[Unset, ParameterDataType]
        if isinstance(_data_type, Unset):
            data_type = UNSET
        else:
            data_type = ParameterDataType(_data_type)

        _display_hint = d.pop("display_hint", UNSET)
        display_hint: Union[Unset, DisplayHint]
        if isinstance(_display_hint, Unset):
            display_hint = UNSET
        else:
            display_hint = DisplayHint(_display_hint)

        required = d.pop("required", UNSET)

        allowed_values = cast(List[str], d.pop("allowed_values", UNSET))

        default_value = d.pop("default_value", UNSET)

        parameter_definition_spec = cls(
            field_name=field_name,
            display_name=display_name,
            help_text=help_text,
            data_type=data_type,
            display_hint=display_hint,
            required=required,
            allowed_values=allowed_values,
            default_value=default_value,
        )

        parameter_definition_spec.additional_properties = d
        return parameter_definition_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_bool_daily_partitioned_checks_spec_custom_checks import (
        ColumnBoolDailyPartitionedChecksSpecCustomChecks,
    )
    from ..models.column_false_percent_check_spec import ColumnFalsePercentCheckSpec
    from ..models.column_true_percent_check_spec import ColumnTruePercentCheckSpec


T = TypeVar("T", bound="ColumnBoolDailyPartitionedChecksSpec")


@_attrs_define
class ColumnBoolDailyPartitionedChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnBoolDailyPartitionedChecksSpecCustomChecks]): Dictionary of additional custom
            checks within this category. The keys are check names defined in the definition section. The sensor parameters
            and rules should match the type of the configured sensor and rule for the custom check.
        daily_partition_true_percent (Union[Unset, ColumnTruePercentCheckSpec]):
        daily_partition_false_percent (Union[Unset, ColumnFalsePercentCheckSpec]):
    """

    custom_checks: Union[Unset, "ColumnBoolDailyPartitionedChecksSpecCustomChecks"] = (
        UNSET
    )
    daily_partition_true_percent: Union[Unset, "ColumnTruePercentCheckSpec"] = UNSET
    daily_partition_false_percent: Union[Unset, "ColumnFalsePercentCheckSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        daily_partition_true_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_partition_true_percent, Unset):
            daily_partition_true_percent = self.daily_partition_true_percent.to_dict()

        daily_partition_false_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_partition_false_percent, Unset):
            daily_partition_false_percent = self.daily_partition_false_percent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if daily_partition_true_percent is not UNSET:
            field_dict["daily_partition_true_percent"] = daily_partition_true_percent
        if daily_partition_false_percent is not UNSET:
            field_dict["daily_partition_false_percent"] = daily_partition_false_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_bool_daily_partitioned_checks_spec_custom_checks import (
            ColumnBoolDailyPartitionedChecksSpecCustomChecks,
        )
        from ..models.column_false_percent_check_spec import ColumnFalsePercentCheckSpec
        from ..models.column_true_percent_check_spec import ColumnTruePercentCheckSpec

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnBoolDailyPartitionedChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = ColumnBoolDailyPartitionedChecksSpecCustomChecks.from_dict(
                _custom_checks
            )

        _daily_partition_true_percent = d.pop("daily_partition_true_percent", UNSET)
        daily_partition_true_percent: Union[Unset, ColumnTruePercentCheckSpec]
        if isinstance(_daily_partition_true_percent, Unset):
            daily_partition_true_percent = UNSET
        else:
            daily_partition_true_percent = ColumnTruePercentCheckSpec.from_dict(
                _daily_partition_true_percent
            )

        _daily_partition_false_percent = d.pop("daily_partition_false_percent", UNSET)
        daily_partition_false_percent: Union[Unset, ColumnFalsePercentCheckSpec]
        if isinstance(_daily_partition_false_percent, Unset):
            daily_partition_false_percent = UNSET
        else:
            daily_partition_false_percent = ColumnFalsePercentCheckSpec.from_dict(
                _daily_partition_false_percent
            )

        column_bool_daily_partitioned_checks_spec = cls(
            custom_checks=custom_checks,
            daily_partition_true_percent=daily_partition_true_percent,
            daily_partition_false_percent=daily_partition_false_percent,
        )

        column_bool_daily_partitioned_checks_spec.additional_properties = d
        return column_bool_daily_partitioned_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.incident_grouping_level import IncidentGroupingLevel
from ..models.minimum_grouping_severity_level import MinimumGroupingSeverityLevel
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.incident_notification_spec import IncidentNotificationSpec


T = TypeVar("T", bound="ConnectionIncidentGroupingSpec")


@_attrs_define
class ConnectionIncidentGroupingSpec:
    """
    Attributes:
        grouping_level (Union[Unset, IncidentGroupingLevel]):
        minimum_severity (Union[Unset, MinimumGroupingSeverityLevel]):
        divide_by_data_groups (Union[Unset, bool]): Create separate data quality incidents for each data group, creating
            different incidents for different groups of rows. By default, data groups are ignored for grouping data quality
            issues into data quality incidents.
        max_incident_length_days (Union[Unset, int]): The maximum length of a data quality incident in days. When a new
            data quality issue is detected after max_incident_length_days days since a similar data quality was first seen,
            a new data quality incident is created that will capture all following data quality issues for the next
            max_incident_length_days days. The default value is 60 days.
        mute_for_days (Union[Unset, int]): The number of days that all similar data quality issues are muted when a a
            data quality incident is closed in the 'mute' status.
        disabled (Union[Unset, bool]): Disables data quality incident creation for failed data quality checks on the
            data source.
        incident_notification (Union[Unset, IncidentNotificationSpec]):
    """

    grouping_level: Union[Unset, IncidentGroupingLevel] = UNSET
    minimum_severity: Union[Unset, MinimumGroupingSeverityLevel] = UNSET
    divide_by_data_groups: Union[Unset, bool] = UNSET
    max_incident_length_days: Union[Unset, int] = UNSET
    mute_for_days: Union[Unset, int] = UNSET
    disabled: Union[Unset, bool] = UNSET
    incident_notification: Union[Unset, "IncidentNotificationSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        grouping_level: Union[Unset, str] = UNSET
        if not isinstance(self.grouping_level, Unset):
            grouping_level = self.grouping_level.value

        minimum_severity: Union[Unset, str] = UNSET
        if not isinstance(self.minimum_severity, Unset):
            minimum_severity = self.minimum_severity.value

        divide_by_data_groups = self.divide_by_data_groups
        max_incident_length_days = self.max_incident_length_days
        mute_for_days = self.mute_for_days
        disabled = self.disabled
        incident_notification: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.incident_notification, Unset):
            incident_notification = self.incident_notification.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if grouping_level is not UNSET:
            field_dict["grouping_level"] = grouping_level
        if minimum_severity is not UNSET:
            field_dict["minimum_severity"] = minimum_severity
        if divide_by_data_groups is not UNSET:
            field_dict["divide_by_data_groups"] = divide_by_data_groups
        if max_incident_length_days is not UNSET:
            field_dict["max_incident_length_days"] = max_incident_length_days
        if mute_for_days is not UNSET:
            field_dict["mute_for_days"] = mute_for_days
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if incident_notification is not UNSET:
            field_dict["incident_notification"] = incident_notification

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.incident_notification_spec import IncidentNotificationSpec

        d = src_dict.copy()
        _grouping_level = d.pop("grouping_level", UNSET)
        grouping_level: Union[Unset, IncidentGroupingLevel]
        if isinstance(_grouping_level, Unset):
            grouping_level = UNSET
        else:
            grouping_level = IncidentGroupingLevel(_grouping_level)

        _minimum_severity = d.pop("minimum_severity", UNSET)
        minimum_severity: Union[Unset, MinimumGroupingSeverityLevel]
        if isinstance(_minimum_severity, Unset):
            minimum_severity = UNSET
        else:
            minimum_severity = MinimumGroupingSeverityLevel(_minimum_severity)

        divide_by_data_groups = d.pop("divide_by_data_groups", UNSET)

        max_incident_length_days = d.pop("max_incident_length_days", UNSET)

        mute_for_days = d.pop("mute_for_days", UNSET)

        disabled = d.pop("disabled", UNSET)

        _incident_notification = d.pop("incident_notification", UNSET)
        incident_notification: Union[Unset, IncidentNotificationSpec]
        if isinstance(_incident_notification, Unset):
            incident_notification = UNSET
        else:
            incident_notification = IncidentNotificationSpec.from_dict(
                _incident_notification
            )

        connection_incident_grouping_spec = cls(
            grouping_level=grouping_level,
            minimum_severity=minimum_severity,
            divide_by_data_groups=divide_by_data_groups,
            max_incident_length_days=max_incident_length_days,
            mute_for_days=mute_for_days,
            disabled=disabled,
            incident_notification=incident_notification,
        )

        connection_incident_grouping_spec.additional_properties = d
        return connection_incident_grouping_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TextBuiltInDateFormats(str, Enum):
    DDMMYYYY = "DD/MM/YYYY"
    MMDDYYYY = "MM/DD/YYYY"
    MONTH_D_YYYY = "Month D, YYYY"
    YYYYMMDD = "YYYY/MM/DD"
    YYYY_MM_DD = "YYYY-MM-DD"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ComparisonCheckResultModel")


@_attrs_define
class ComparisonCheckResultModel:
    """The table comparison check result model for the most recent data comparison run. Identifies the check name and the
    number of data groupings that passed or failed the comparison.

        Attributes:
            check_name (Union[Unset, str]): DQOps data quality check name.
            executed_at (Union[Unset, int]): The timestamp when the check was executed.
            valid_results (Union[Unset, int]): The number of data groups that were compared and the values matched within
                the accepted error margin for all check severity levels.
            warnings (Union[Unset, int]): The number of data groups that were compared and the values did not match, raising
                a warning severity level data quality issue.
            errors (Union[Unset, int]): The number of data groups that were compared and the values did not match, raising
                an error severity level data quality issue.
            fatals (Union[Unset, int]): The number of data groups that were compared and the values did not match, raising a
                fatal severity level data quality issue.
            execution_errors (Union[Unset, int]): The number of execution errors in the check or rule that prevented
                comparing the tables.
            not_matching_data_groups (Union[Unset, List[str]]): A list of not matching data grouping names.
    """

    check_name: Union[Unset, str] = UNSET
    executed_at: Union[Unset, int] = UNSET
    valid_results: Union[Unset, int] = UNSET
    warnings: Union[Unset, int] = UNSET
    errors: Union[Unset, int] = UNSET
    fatals: Union[Unset, int] = UNSET
    execution_errors: Union[Unset, int] = UNSET
    not_matching_data_groups: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        check_name = self.check_name
        executed_at = self.executed_at
        valid_results = self.valid_results
        warnings = self.warnings
        errors = self.errors
        fatals = self.fatals
        execution_errors = self.execution_errors
        not_matching_data_groups: Union[Unset, List[str]] = UNSET
        if not isinstance(self.not_matching_data_groups, Unset):
            not_matching_data_groups = self.not_matching_data_groups

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if check_name is not UNSET:
            field_dict["check_name"] = check_name
        if executed_at is not UNSET:
            field_dict["executed_at"] = executed_at
        if valid_results is not UNSET:
            field_dict["valid_results"] = valid_results
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if errors is not UNSET:
            field_dict["errors"] = errors
        if fatals is not UNSET:
            field_dict["fatals"] = fatals
        if execution_errors is not UNSET:
            field_dict["execution_errors"] = execution_errors
        if not_matching_data_groups is not UNSET:
            field_dict["not_matching_data_groups"] = not_matching_data_groups

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        check_name = d.pop("check_name", UNSET)

        executed_at = d.pop("executed_at", UNSET)

        valid_results = d.pop("valid_results", UNSET)

        warnings = d.pop("warnings", UNSET)

        errors = d.pop("errors", UNSET)

        fatals = d.pop("fatals", UNSET)

        execution_errors = d.pop("execution_errors", UNSET)

        not_matching_data_groups = cast(
            List[str], d.pop("not_matching_data_groups", UNSET)
        )

        comparison_check_result_model = cls(
            check_name=check_name,
            executed_at=executed_at,
            valid_results=valid_results,
            warnings=warnings,
            errors=errors,
            fatals=fatals,
            execution_errors=execution_errors,
            not_matching_data_groups=not_matching_data_groups,
        )

        comparison_check_result_model.additional_properties = d
        return comparison_check_result_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os
import ssl
import sys
import zipfile

import httpx
import jdk

# ignore those, they are filled by importing version.py
VERSION = "filled by dqops/version.py"
PIP_VERSION = "filled by dqops/version.py"
GITHUB_RELEASE = "filled by dqops/version.py"
JAVA_VERSION = "filled by dqops/version.py"

try:
    exec(
        open(
            os.path.join(os.path.dirname(os.path.realpath(__file__)), "version.py")
        ).read()
    )
except IOError:
    print("Failed to load DQOps version file.", file=sys.stderr)
    sys.exit(-1)


def install_dqo(dest: str, dqo_tag: str, dqo_version: str):
    """
    Downloads and installs DQOps distribution locally.
    :param dest: Destination folder to download and extract the zip package.
    :param dqo_tag: GitHub release tag.
    :param dqo_version:  DQOps version.
    """

    ssl._create_default_https_context = ssl._create_unverified_context

    distribution_local_name = os.path.join(
        dest, "dqo-distribution-%s-bin.zip" % dqo_version
    )

    os.makedirs(dest, exist_ok=True)

    if os.path.exists(distribution_local_name):
        os.remove(distribution_local_name)

    download_to_file(dqo_tag, dqo_version, distribution_local_name)

    with zipfile.ZipFile(distribution_local_name, "r") as zip_ref:
        os_platform = sys.platform.lower()[0:3]
        dest_path = ("\\\\?\\" if os_platform == "win" else "") + os.path.abspath(dest)
        zip_ref.extractall(dest_path)

    if os.path.exists(distribution_local_name):
        os.remove(distribution_local_name)


def http_client() -> httpx.Client:
    return httpx.Client(
        http2=True,
        timeout=httpx.Timeout(5.0, connect=10.0, read=6.0),
        follow_redirects=True,
        headers={
            "Accept-Encoding": "identity",
            "Accept-Language": "en-US,en;q=0.9,pl;q=0.8",
            "Cache-Control": "no-cache, no-transform",
            "Pragma": "no-cache",
            "Sec-Ch-Ua": '"Not_A Brand";v="8", "Chromium";v="120", "Google Chrome";v="120"',
            "Sec-Ch-Ua-Mobile": "?0",
            "Sec-Ch-Ua-Platform": '"Windows"',
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "cross-site",
            "Sec-Fetch-User": "?1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            + "Chrome/120.0.0.0 Safari/537.36",
        },
    )


def download_to_file(dqo_tag: str, dqo_version: str, distribution_local_name: str):
    github_url = (
        "https://github.com/dqops/dqo/releases/download/%s/dqo-distribution-%s-bin.zip"
        % (dqo_tag, dqo_version)
    )

    with http_client() as client:
        if download_to_file_try(
            client, github_url, distribution_local_name, dqo_version
        ):
            return

        print("Download from %s failed, attempting another mirror." % github_url)

        if download_to_file_fallback(client, dqo_version, distribution_local_name):
            return

    print(
        "Download failed. Check your internet connection and firewall settings.",
        file=sys.stderr,
    )
    sys.exit(-1)


def download_to_file_fallback(
    client: httpx.Client, dqo_version: str, distribution_local_name: str
):
    # Set any cookies sent by the resource server.
    bucket_url = "https://dqops.com/releases/dqo-distribution-%s-bin.zip" % dqo_version
    head_url = bucket_url
    client.head(head_url)

    return download_to_file_try(
        client, bucket_url, distribution_local_name, dqo_version
    )


def download_to_file_try(client, source_url, path, dqo_version):
    try:
        trying_message = "Trying to download DQOps version %s from %s" % (
            dqo_version,
            source_url,
        )
        print(trying_message)

        return download_to_file_once(client, source_url, path)
    except Exception as e:
        print(
            "%s: Download interrupted, reason: %s" % (e.__class__.__name__, e),
            file=sys.stderr,
        )

    return False


def download_to_file_once(client, source_url, path, chunk_size=1024 * 1024):
    with open(path, mode="wb") as dest:
        new_bytes_so_far = 0
        while True:
            bytes_so_far = new_bytes_so_far
            start_byte = bytes_so_far if bytes_so_far > 0 else None
            new_bytes_so_far, total_size = download_to_file_partial(
                client, source_url, dest, chunk_size=chunk_size, start_byte=start_byte
            )

            if new_bytes_so_far == bytes_so_far:
                print("No progress since last download attempt, abort.")
                return False
            elif new_bytes_so_far == total_size:
                return True


def download_to_file_partial(
    client, source_url, opened_dest, chunk_size=1024 * 1024, start_byte: int = None
):
    if start_byte and start_byte != 0:
        bytes_so_far = start_byte
        headers = {"Range": f"bytes={start_byte}-"}
    else:
        bytes_so_far = 0
        headers = None

    try:
        with client.stream("GET", source_url, headers=headers) as response_stream:
            total_size = int(response_stream.headers.get("Content-Length").strip()) + (
                start_byte if start_byte else 0
            )

            if (
                start_byte
                and start_byte != 0
                and "Content-Range" not in response_stream.headers
            ):
                # Partial download not allowed, shouldn't attempt further retries.
                return bytes_so_far, total_size

            for chunk in response_stream.iter_raw(chunk_size=chunk_size):
                if not chunk:
                    return bytes_so_far, total_size

                chunk_bytes = len(chunk)
                opened_dest.write(chunk)
                bytes_so_far += chunk_bytes

                print(
                    "Downloaded %d of %d bytes (%0.2f%%)"
                    % (
                        bytes_so_far,
                        total_size,
                        round(float(bytes_so_far) / total_size * 100, 2),
                    )
                )

    except httpx.TimeoutException:
        pass

    return bytes_so_far, total_size


def install_dqo_home_if_missing(dqo_home):
    if os.path.exists(dqo_home) and os.path.exists(os.path.join(dqo_home, "README.md")):
        return

    install_dqo(dqo_home, GITHUB_RELEASE, VERSION)


def install_jre_if_missing(java_install_dir):
    if os.path.exists(java_install_dir) and len(os.listdir(java_install_dir)) > 0:
        return

    ssl._create_default_https_context = ssl._create_unverified_context

    print("Installing Java JRE %s at %s" % (JAVA_VERSION, java_install_dir))
    jdk.install(JAVA_VERSION, jre=True, path=java_install_dir)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.dqo_user_role import DqoUserRole
from ..types import UNSET, Unset

T = TypeVar("T", bound="DqoUserProfileModel")


@_attrs_define
class DqoUserProfileModel:
    """The model that describes the current user and his access rights.

    Attributes:
        user (Union[Unset, str]): User email.
        tenant (Union[Unset, str]): DQOps Cloud tenant.
        license_type (Union[Unset, str]): DQOps Cloud license type.
        trial_period_expires_at (Union[Unset, str]): The date and time when the trial period of a PERSONAL DQOps license
            expires and the account is downgraded to a FREE license.
        connections_limit (Union[Unset, int]): Limit of the number of connections that can be synchronized to the DQOps
            Cloud data quality warehouse.
        users_limit (Union[Unset, int]): Limit of the number of users that can be added to a DQOps environment.
        months_limit (Union[Unset, int]): Limit of the number of recent months (excluding the current month) that can be
            synchronized to the DQOps Cloud data quality warehouse.
        connection_tables_limit (Union[Unset, int]): Limit of the number of tables inside each connection that can be
            synchronized to the DQOps Cloud data quality warehouse.
        tables_limit (Union[Unset, int]): Limit of the total number of tables that can be synchronized to the DQOps
            Cloud data quality warehouse.
        jobs_limit (Union[Unset, int]): Limit of the number of supported concurrent jobs that DQOps can run in parallel
            on this instance.
        data_domains_limit (Union[Unset, int]): Limit of the number of supported data domains in an ENTERPRISE version
            of DQOps.
        account_role (Union[Unset, DqoUserRole]):
        data_quality_data_warehouse_enabled (Union[Unset, bool]): True when the account has access to the DQOps Cloud's
            data quality data lake and data warehouse, allowing to synchronize files and use the data quality data
            warehouse.
        can_manage_account (Union[Unset, bool]): User is the administrator of the account and can perform security
            related actions, such as managing users.
        can_view_any_object (Union[Unset, bool]): User can view any object and view all results.
        can_manage_scheduler (Union[Unset, bool]): User can start and stop the job scheduler.
        can_cancel_jobs (Union[Unset, bool]): User can cancel running jobs.
        can_run_checks (Union[Unset, bool]): User can run data quality checks.
        can_delete_data (Union[Unset, bool]): User can delete data quality results.
        can_collect_statistics (Union[Unset, bool]): User can collect statistics.
        can_manage_data_sources (Union[Unset, bool]): User can manage data sources: create connections, import tables,
            change the configuration of connections, tables, columns. Change any settings in the Data Sources section.
        can_synchronize (Union[Unset, bool]): User can trigger the synchronization with DQOps Cloud.
        can_edit_comments (Union[Unset, bool]): User can edit comments on connections, tables, columns.
        can_edit_labels (Union[Unset, bool]): User can edit labels on connections, tables, columns.
        can_manage_definitions (Union[Unset, bool]): User can manage definitions of sensors, rules, checks and the
            default data quality check configuration that is applied on imported tables.
        can_compare_tables (Union[Unset, bool]): User can define table comparison configurations and compare tables.
        can_manage_users (Union[Unset, bool]): User can manage other users, add users to a multi-user account, change
            access rights, reset passwords.
        can_manage_and_view_shared_credentials (Union[Unset, bool]): User can manage shared credentials and view (or
            download) already defined shared credentials.
        can_change_own_password (Union[Unset, bool]): User can change his own password in DQOps Cloud, because the DQOps
            Cloud Pairing API Key is valid and synchronization is enabled.
        can_use_data_domains (Union[Unset, bool]): User can use data domains. Support for data domains requires an
            ENTERPRISE license of DQOps.
        can_synchronize_to_data_catalog (Union[Unset, bool]): User can synchronize data to a data catalog. The instance
            must be configured correctly and the user must have at least an EDITOR role.
        can_use_ai_anomaly_detection (Union[Unset, bool]): The DQOps instance is a paid version with advanced AI anomaly
            prediction.
        can_logout (Union[Unset, bool]): This instance uses federated authentication and the user can log out.
    """

    user: Union[Unset, str] = UNSET
    tenant: Union[Unset, str] = UNSET
    license_type: Union[Unset, str] = UNSET
    trial_period_expires_at: Union[Unset, str] = UNSET
    connections_limit: Union[Unset, int] = UNSET
    users_limit: Union[Unset, int] = UNSET
    months_limit: Union[Unset, int] = UNSET
    connection_tables_limit: Union[Unset, int] = UNSET
    tables_limit: Union[Unset, int] = UNSET
    jobs_limit: Union[Unset, int] = UNSET
    data_domains_limit: Union[Unset, int] = UNSET
    account_role: Union[Unset, DqoUserRole] = UNSET
    data_quality_data_warehouse_enabled: Union[Unset, bool] = UNSET
    can_manage_account: Union[Unset, bool] = UNSET
    can_view_any_object: Union[Unset, bool] = UNSET
    can_manage_scheduler: Union[Unset, bool] = UNSET
    can_cancel_jobs: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    can_collect_statistics: Union[Unset, bool] = UNSET
    can_manage_data_sources: Union[Unset, bool] = UNSET
    can_synchronize: Union[Unset, bool] = UNSET
    can_edit_comments: Union[Unset, bool] = UNSET
    can_edit_labels: Union[Unset, bool] = UNSET
    can_manage_definitions: Union[Unset, bool] = UNSET
    can_compare_tables: Union[Unset, bool] = UNSET
    can_manage_users: Union[Unset, bool] = UNSET
    can_manage_and_view_shared_credentials: Union[Unset, bool] = UNSET
    can_change_own_password: Union[Unset, bool] = UNSET
    can_use_data_domains: Union[Unset, bool] = UNSET
    can_synchronize_to_data_catalog: Union[Unset, bool] = UNSET
    can_use_ai_anomaly_detection: Union[Unset, bool] = UNSET
    can_logout: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user = self.user
        tenant = self.tenant
        license_type = self.license_type
        trial_period_expires_at = self.trial_period_expires_at
        connections_limit = self.connections_limit
        users_limit = self.users_limit
        months_limit = self.months_limit
        connection_tables_limit = self.connection_tables_limit
        tables_limit = self.tables_limit
        jobs_limit = self.jobs_limit
        data_domains_limit = self.data_domains_limit
        account_role: Union[Unset, str] = UNSET
        if not isinstance(self.account_role, Unset):
            account_role = self.account_role.value

        data_quality_data_warehouse_enabled = self.data_quality_data_warehouse_enabled
        can_manage_account = self.can_manage_account
        can_view_any_object = self.can_view_any_object
        can_manage_scheduler = self.can_manage_scheduler
        can_cancel_jobs = self.can_cancel_jobs
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data
        can_collect_statistics = self.can_collect_statistics
        can_manage_data_sources = self.can_manage_data_sources
        can_synchronize = self.can_synchronize
        can_edit_comments = self.can_edit_comments
        can_edit_labels = self.can_edit_labels
        can_manage_definitions = self.can_manage_definitions
        can_compare_tables = self.can_compare_tables
        can_manage_users = self.can_manage_users
        can_manage_and_view_shared_credentials = (
            self.can_manage_and_view_shared_credentials
        )
        can_change_own_password = self.can_change_own_password
        can_use_data_domains = self.can_use_data_domains
        can_synchronize_to_data_catalog = self.can_synchronize_to_data_catalog
        can_use_ai_anomaly_detection = self.can_use_ai_anomaly_detection
        can_logout = self.can_logout

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if tenant is not UNSET:
            field_dict["tenant"] = tenant
        if license_type is not UNSET:
            field_dict["license_type"] = license_type
        if trial_period_expires_at is not UNSET:
            field_dict["trial_period_expires_at"] = trial_period_expires_at
        if connections_limit is not UNSET:
            field_dict["connections_limit"] = connections_limit
        if users_limit is not UNSET:
            field_dict["users_limit"] = users_limit
        if months_limit is not UNSET:
            field_dict["months_limit"] = months_limit
        if connection_tables_limit is not UNSET:
            field_dict["connection_tables_limit"] = connection_tables_limit
        if tables_limit is not UNSET:
            field_dict["tables_limit"] = tables_limit
        if jobs_limit is not UNSET:
            field_dict["jobs_limit"] = jobs_limit
        if data_domains_limit is not UNSET:
            field_dict["data_domains_limit"] = data_domains_limit
        if account_role is not UNSET:
            field_dict["account_role"] = account_role
        if data_quality_data_warehouse_enabled is not UNSET:
            field_dict["data_quality_data_warehouse_enabled"] = (
                data_quality_data_warehouse_enabled
            )
        if can_manage_account is not UNSET:
            field_dict["can_manage_account"] = can_manage_account
        if can_view_any_object is not UNSET:
            field_dict["can_view_any_object"] = can_view_any_object
        if can_manage_scheduler is not UNSET:
            field_dict["can_manage_scheduler"] = can_manage_scheduler
        if can_cancel_jobs is not UNSET:
            field_dict["can_cancel_jobs"] = can_cancel_jobs
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data
        if can_collect_statistics is not UNSET:
            field_dict["can_collect_statistics"] = can_collect_statistics
        if can_manage_data_sources is not UNSET:
            field_dict["can_manage_data_sources"] = can_manage_data_sources
        if can_synchronize is not UNSET:
            field_dict["can_synchronize"] = can_synchronize
        if can_edit_comments is not UNSET:
            field_dict["can_edit_comments"] = can_edit_comments
        if can_edit_labels is not UNSET:
            field_dict["can_edit_labels"] = can_edit_labels
        if can_manage_definitions is not UNSET:
            field_dict["can_manage_definitions"] = can_manage_definitions
        if can_compare_tables is not UNSET:
            field_dict["can_compare_tables"] = can_compare_tables
        if can_manage_users is not UNSET:
            field_dict["can_manage_users"] = can_manage_users
        if can_manage_and_view_shared_credentials is not UNSET:
            field_dict["can_manage_and_view_shared_credentials"] = (
                can_manage_and_view_shared_credentials
            )
        if can_change_own_password is not UNSET:
            field_dict["can_change_own_password"] = can_change_own_password
        if can_use_data_domains is not UNSET:
            field_dict["can_use_data_domains"] = can_use_data_domains
        if can_synchronize_to_data_catalog is not UNSET:
            field_dict["can_synchronize_to_data_catalog"] = (
                can_synchronize_to_data_catalog
            )
        if can_use_ai_anomaly_detection is not UNSET:
            field_dict["can_use_ai_anomaly_detection"] = can_use_ai_anomaly_detection
        if can_logout is not UNSET:
            field_dict["can_logout"] = can_logout

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user = d.pop("user", UNSET)

        tenant = d.pop("tenant", UNSET)

        license_type = d.pop("license_type", UNSET)

        trial_period_expires_at = d.pop("trial_period_expires_at", UNSET)

        connections_limit = d.pop("connections_limit", UNSET)

        users_limit = d.pop("users_limit", UNSET)

        months_limit = d.pop("months_limit", UNSET)

        connection_tables_limit = d.pop("connection_tables_limit", UNSET)

        tables_limit = d.pop("tables_limit", UNSET)

        jobs_limit = d.pop("jobs_limit", UNSET)

        data_domains_limit = d.pop("data_domains_limit", UNSET)

        _account_role = d.pop("account_role", UNSET)
        account_role: Union[Unset, DqoUserRole]
        if isinstance(_account_role, Unset):
            account_role = UNSET
        else:
            account_role = DqoUserRole(_account_role)

        data_quality_data_warehouse_enabled = d.pop(
            "data_quality_data_warehouse_enabled", UNSET
        )

        can_manage_account = d.pop("can_manage_account", UNSET)

        can_view_any_object = d.pop("can_view_any_object", UNSET)

        can_manage_scheduler = d.pop("can_manage_scheduler", UNSET)

        can_cancel_jobs = d.pop("can_cancel_jobs", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        can_collect_statistics = d.pop("can_collect_statistics", UNSET)

        can_manage_data_sources = d.pop("can_manage_data_sources", UNSET)

        can_synchronize = d.pop("can_synchronize", UNSET)

        can_edit_comments = d.pop("can_edit_comments", UNSET)

        can_edit_labels = d.pop("can_edit_labels", UNSET)

        can_manage_definitions = d.pop("can_manage_definitions", UNSET)

        can_compare_tables = d.pop("can_compare_tables", UNSET)

        can_manage_users = d.pop("can_manage_users", UNSET)

        can_manage_and_view_shared_credentials = d.pop(
            "can_manage_and_view_shared_credentials", UNSET
        )

        can_change_own_password = d.pop("can_change_own_password", UNSET)

        can_use_data_domains = d.pop("can_use_data_domains", UNSET)

        can_synchronize_to_data_catalog = d.pop(
            "can_synchronize_to_data_catalog", UNSET
        )

        can_use_ai_anomaly_detection = d.pop("can_use_ai_anomaly_detection", UNSET)

        can_logout = d.pop("can_logout", UNSET)

        dqo_user_profile_model = cls(
            user=user,
            tenant=tenant,
            license_type=license_type,
            trial_period_expires_at=trial_period_expires_at,
            connections_limit=connections_limit,
            users_limit=users_limit,
            months_limit=months_limit,
            connection_tables_limit=connection_tables_limit,
            tables_limit=tables_limit,
            jobs_limit=jobs_limit,
            data_domains_limit=data_domains_limit,
            account_role=account_role,
            data_quality_data_warehouse_enabled=data_quality_data_warehouse_enabled,
            can_manage_account=can_manage_account,
            can_view_any_object=can_view_any_object,
            can_manage_scheduler=can_manage_scheduler,
            can_cancel_jobs=can_cancel_jobs,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
            can_collect_statistics=can_collect_statistics,
            can_manage_data_sources=can_manage_data_sources,
            can_synchronize=can_synchronize,
            can_edit_comments=can_edit_comments,
            can_edit_labels=can_edit_labels,
            can_manage_definitions=can_manage_definitions,
            can_compare_tables=can_compare_tables,
            can_manage_users=can_manage_users,
            can_manage_and_view_shared_credentials=can_manage_and_view_shared_credentials,
            can_change_own_password=can_change_own_password,
            can_use_data_domains=can_use_data_domains,
            can_synchronize_to_data_catalog=can_synchronize_to_data_catalog,
            can_use_ai_anomaly_detection=can_use_ai_anomaly_detection,
            can_logout=can_logout,
        )

        dqo_user_profile_model.additional_properties = d
        return dqo_user_profile_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
